@@ -32,7 +32,8 @@
 #include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 
-#include <QString>
+#include <QFile>
+#include <QIODevice>
 
 namespace CMakeProjectManager {
 
@@ -43,8 +44,9 @@ namespace CMakeProjectManager {
 CMakeConfigItem::CMakeConfigItem() = default;
 
 CMakeConfigItem::CMakeConfigItem(const QByteArray &k, Type t,
-                                 const QByteArray &d, const QByteArray &v) :
-    key(k), type(t), value(v), documentation(d)
+                                 const QByteArray &d, const QByteArray &v,
+                                 const QStringList &s) :
+    key(k), type(t), value(v), documentation(d), values(s)
 { }
 
 CMakeConfigItem::CMakeConfigItem(const QByteArray &k, const QByteArray &v) :
@@ -143,9 +145,52 @@ CMakeConfigItem::Type CMakeConfigItem::typeStringToType(const QByteArray &type)
         return CMakeConfigItem::PATH;
     if (type == "STATIC")
         return CMakeConfigItem::STATIC;
+    if (type == "INTERNAL")
+        return CMakeConfigItem::INTERNAL;
 
-    QTC_CHECK(type == "INTERNAL" || type == "UNINITIALIZED");
-    return CMakeConfigItem::INTERNAL;
+    QTC_CHECK(type == "UNINITIALIZED");
+    return CMakeConfigItem::UNINITIALIZED;
+}
+
+QString CMakeConfigItem::typeToTypeString(const CMakeConfigItem::Type t)
+{
+    switch (t) {
+    case CMakeProjectManager::CMakeConfigItem::FILEPATH:
+        return {"FILEPATH"};
+    case CMakeProjectManager::CMakeConfigItem::PATH:
+        return {"PATH"};
+    case CMakeProjectManager::CMakeConfigItem::STRING:
+        return {"STRING"};
+    case CMakeProjectManager::CMakeConfigItem::INTERNAL:
+        return {"INTERNAL"};
+    case CMakeProjectManager::CMakeConfigItem::STATIC:
+        return {"STATIC"};
+    case CMakeProjectManager::CMakeConfigItem::BOOL:
+        return {"BOOL"};
+    case CMakeProjectManager::CMakeConfigItem::UNINITIALIZED:
+        return {"UNINITIALIZED"};
+    }
+    QTC_CHECK(false);
+    return {};
+}
+
+Utils::optional<bool> CMakeConfigItem::toBool(const QByteArray &value)
+{
+    // Taken from CMakes if(<constant>) documentation:
+    // "Named boolean constants are case-insensitive."
+    const QString v = QString::fromUtf8(value).toUpper();
+
+    bool isInt = false;
+    v.toInt(&isInt);
+
+    // "False if the constant is 0, OFF, NO, FALSE, N, IGNORE, NOTFOUND, the empty string, or ends in the suffix -NOTFOUND."
+    if (v == "0" || v == "OFF" || v == "NO" || v == "FALSE" || v == "N" || v == "IGNORE" || v == "NOTFOUND" || v == "" || v.endsWith("-NOTFOUND"))
+        return false;
+    // "True if the constant is 1, ON, YES, TRUE, Y, or a non-zero number."
+    if (v == "1" || v == "ON" || v == "YES" || v == "TRUE" || v == "Y" || isInt)
+        return true;
+
+    return {};
 }
 
 QString CMakeConfigItem::expandedValue(const ProjectExplorer::Kit *k) const
@@ -155,7 +200,7 @@ QString CMakeConfigItem::expandedValue(const ProjectExplorer::Kit *k) const
 
 QString CMakeConfigItem::expandedValue(const Utils::MacroExpander *expander) const
 {
-    return expander->expand(QString::fromUtf8(value));
+    return expander ? expander->expand(QString::fromUtf8(value)) : QString::fromUtf8(value);
 }
 
 std::function<bool (const CMakeConfigItem &a, const CMakeConfigItem &b)> CMakeConfigItem::sortOperator()
@@ -243,6 +288,54 @@ static QByteArrayList splitCMakeCacheLine(const QByteArray &line) {
                             << line.mid(equalPos + 1);
 }
 
+static CMakeConfigItem setItemFromString(const QString &input)
+{
+    return CMakeConfigItem::fromString(input);
+}
+
+static CMakeConfigItem unsetItemFromString(const QString &input)
+{
+    CMakeConfigItem item(input.toUtf8(), QByteArray());
+    item.isUnset = true;
+    return item;
+}
+
+QList<CMakeConfigItem> CMakeConfigItem::itemsFromArguments(const QStringList &list)
+{
+    CMakeConfig result;
+    bool inSet = false;
+    bool inUnset = false;
+    for (const QString &i : list) {
+        if (inSet) {
+            inSet = false;
+            result.append(setItemFromString(i));
+            continue;
+        }
+        if (inUnset) {
+            inUnset = false;
+            result.append(unsetItemFromString(i));
+            continue;
+        }
+        if (i == "-U") {
+            inUnset = true;
+            continue;
+        }
+        if (i == "-D") {
+            inSet = true;
+            continue;
+        }
+        if (i.startsWith("-U")) {
+            result.append(unsetItemFromString(i.mid(2)));
+            continue;
+        }
+        if (i.startsWith("-D")) {
+            result.append(setItemFromString(i.mid(2)));
+        }
+        // ignore everything else as that does not define a configuration option
+    }
+    return result;
+}
+
 QList<CMakeConfigItem> CMakeConfigItem::itemsFromFile(const Utils::FilePath &cacheFile, QString *errorMessage)
 {
     CMakeConfig result;
@@ -329,6 +422,9 @@ QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
     case CMakeProjectManager::CMakeConfigItem::INTERNAL:
         typeStr = QLatin1String("INTERNAL");
         break;
+    case CMakeProjectManager::CMakeConfigItem::UNINITIALIZED:
+        typeStr = QLatin1String("UNINITIALIZED");
+        break;
     case CMakeProjectManager::CMakeConfigItem::STRING:
     default:
         typeStr = QLatin1String("STRING");
@@ -347,6 +443,18 @@ QString CMakeConfigItem::toArgument(const Utils::MacroExpander *expander) const
     return "-D" + toString(expander);
 }
 
+QString CMakeConfigItem::toCMakeSetLine(const Utils::MacroExpander *expander) const
+{
+    if (isUnset) {
+        return QString("unset(\"%1\" CACHE)").arg(QString::fromUtf8(key));
+    }
+    return QString("set(\"%1\" \"%2\" CACHE \"%3\" \"%4\" FORCE)")
+        .arg(QString::fromUtf8(key))
+        .arg(expandedValue(expander))
+        .arg(typeToTypeString(type))
+        .arg(QString::fromUtf8(documentation));
+}
+
 bool CMakeConfigItem::operator==(const CMakeConfigItem &o) const
 {
     // type, isAdvanced and documentation do not matter for a match!
